!     MEX-file for computing the Hill matrix
!     S. Karkar, A.Lazarus
!     September 2010

!     usage : Hill = Hill_mat (syshb_object, ddlstab, nbddlstab, nbddl, H)

#include <fintrf_new.h>

module Hill_mod

  use jacobien

!    !
!    ! interfaces with Jacobian functions
!    !
!  interface
!    function J0(lambda,omega)
!     real(8) lambda,omega
!     complex(8) J0(nbddl,nbddl)
!    end function J0
!    function JL(lambda,omega,Ustab)
!     real(8) lambda,omega
!     complex(8) Ustab(nbddl), JL(nbddl,nbddl)
!    end function JL
!    function JQ(lambda,omega,Ustab,Vstab)
!     real(8) lambda,omega
!     complex(8) Ustab(nbddl), Vstab(nbddl), JQ(nbddl,nbddl)
!    end function JQ
!  end interface

  contains
    subroutine calc_Hill(ddlstab_pr,N,HH,nbddlstab,HILL_pr, HILL_pi)

      implicit none
      integer, intent(in) :: HH, nbddlstab
      mwSize N
      mwPointer :: HILL_pr,HILL_pi, ddlstab_pr
      double precision :: ddlstab(N)
      integer :: i,h,hm,hp,p,pn,pp,ind,ind0,opp,opp0,g,gm,go
      double precision :: Ident(nbddl,nbddl), UU(nbddlstab), UH(2*HH*nbddlstab), omega, lambda,&
 zer_Ustab(nbddlstab), zer_Id(nbddl,nbddl)
      complex(8) :: JJ(nbddl*(2*HH+1),nbddl), Ustab(nbddlstab), Vstab(nbddlstab)
      complex(8) :: HILL(nbddl*(2*HH+1),nbddl*(2*HH+1)), Hill_lin(nbddl*nbddl*(2*HH+1)*(2*HH+1))

      call mxCopyPtrToReal8(ddlstab_pr,ddlstab,N)

! initialize output Hill matrix
      HILL = 0d0

! zero vector and matrix
      zer_Ustab=0d0
      zer_Id=0d0

! Identity matrix, size nbddl
      Ident(:,:) = 0d0
      do i=1,nbddl
        Ident(i,i)=1d0
      end do

      JJ = 0d0

      UU = ddlstab(1:nbddlstab)
      UH = ddlstab(nbddlstab+1:N-2)
      omega = ddlstab(N)
      lambda = ddlstab(N-1)

      do h = -HH,HH

        hm = abs(h) - 1
        hp = abs(h)
        
        if (h .eq. 0) then
          Ustab = cmplx(UU,zer_Ustab,8)
        else if (h .lt. 0) then
          Ustab = cmplx(0.5d0*UH(nbddlstab*2*hm+1:nbddlstab*(2*hp-1)),&
                                                     0.5d0*UH(nbddlstab*(2*hm+1)+1:nbddlstab*2*hp),8)
        else
          Ustab = cmplx(0.5d0*UH(nbddlstab*2*hm+1:nbddlstab*(2*hp-1)),&
                                                     -0.5d0*UH(nbddlstab*(2*hm+1)+1:nbddlstab*2*hp),8)
        end if


        if (sum(abs(JQ(lambda,omega,Ustab,Ustab))) /= 0d0) then
        do p = h,HH

          pn = abs(p) - 1
          pp = abs(p)


          if (abs(p+h) .lt. HH+1) then

            if (p .eq. 0) then
            Vstab = UU
            else if (p .lt. 0) then
              Vstab = cmplx(0.5d0*UH(nbddlstab*2*pn+1:nbddlstab*(2*pp-1)),&
                                                     0.5d0*UH(nbddlstab*(2*pn+1)+1:nbddlstab*2*pp),8)
            else
              Vstab = cmplx(0.5d0*UH(nbddlstab*2*pn+1:nbddlstab*(2*pp-1)),&
                                                     -0.5d0*UH(nbddlstab*(2*pn+1)+1:nbddlstab*2*pp),8)
            end if    

            ind = p+h + (HH+1)
            ind0 = ind - 1
            opp = -(p+h) + (HH+1)
            opp0 = opp - 1

            if (p .eq. h) then
              JJ(ind0*nbddl+1:ind*nbddl,1:nbddl) = JJ(ind0*nbddl+1:ind*nbddl,1:nbddl) + JQ(lambda,omega,Ustab,Vstab)
            else
              JJ(ind0*nbddl+1:ind*nbddl,1:nbddl) = JJ(ind0*nbddl+1:ind*nbddl,1:nbddl) + JQ(lambda,omega,Ustab,Vstab)
              JJ(opp0*nbddl+1:opp*nbddl,1:nbddl) = JJ(opp0*nbddl+1:opp*nbddl,1:nbddl) + JQ(lambda,omega,conjg(Ustab),conjg(Vstab))
            end if    
          end if    ! if abs(p+h)...             
        end do   ! do p=...
        end if ! JQ/=0
            
        if (h < 1) then
            if (h == 0) then
                JJ((h+HH)*nbddl+1:(h+HH+1)*nbddl,1:nbddl) = JJ((h+HH)*nbddl+1:(h+HH+1)*nbddl,1:nbddl)&
                                                                                        + J0(lambda,omega) + JL(lambda,omega,Ustab)
            else
                JJ((h+HH)*nbddl+1:(h+HH+1)*nbddl,1:nbddl) = JJ((h+HH)*nbddl+1:(h+HH+1)*nbddl,1:nbddl) + JL(lambda,omega,Ustab)
                JJ((-h+HH)*nbddl+1:(-h+HH+1)*nbddl,1:nbddl) = JJ((-h+HH)*nbddl+1:(-h+HH+1)*nbddl,1:nbddl)&
                                                                                                 + JL(lambda,omega,conjg(Ustab))
            end if
        end if
            
      end do ! h=...
        
        
      do g = 1,HH
        gm = g - 1;
        go = 2*HH - gm;
        HILL(1:(HH+g)*nbddl,gm*nbddl+1:g*nbddl) = JJ((HH-gm)*nbddl+1:nbddl*(2*HH+1),:)
        HILL(gm*nbddl+1:g*nbddl,gm*nbddl+1:g*nbddl) = HILL(gm*nbddl+1:g*nbddl,gm*nbddl+1:g*nbddl)&
                                                                                            + cmplx(zer_Id,omega*Ident*(HH - gm),8)
        HILL((HH-gm)*nbddl+1:nbddl*(2*HH+1),go*nbddl+1:(go+1)*nbddl) = JJ(1:(HH+g)*nbddl,:)
        HILL(go*nbddl+1:(go+1)*nbddl,go*nbddl+1:(go+1)*nbddl) = HILL(go*nbddl+1:(go+1)*nbddl,go*nbddl+1:(go+1)*nbddl)&
                                                                                            + cmplx(zer_Id,omega*Ident*(gm - HH),8)
      end do
            
        HILL(:,HH*nbddl+1:(HH+1)*nbddl) = JJ
        call mxCopyComplex16ToPtr(HILL,HILL_pr,Hill_pi,nbddl*(2*HH+1)*nbddl*(2*HH+1))

    end subroutine calc_Hill

end module Hill_mod



subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    use Hill_mod
    implicit none
    integer(4) nlhs, nrhs
    mwPointer plhs(nlhs), prhs(nrhs), mxGetPr, mxGetPi, mxCreateDoubleMatrix, tmp_pr, ddlstab_pr, HILL_pr, Hill_pi
    integer :: HH, nbddlstab, nbddl_test
    double precision :: tmp
    mwSize mm,nn, mxGetN, mxGetM, N
    character*(*), parameter :: err1='Hill_mat requires 5 input arguments : syshb_obj, ddlstab, nbddlstab, nbddl, H.',&
err2='Hill_mat requires 1 output arguments : HILL.', err3='ddlstab must be a vector.', &
err4='ddlstab length must be : N = (2*H+1)*nbddlstab+2.', err5='nbddl mismatch.'

! argument numbers test
    if (nrhs /= 5) call mexerrmsgtxt(err1)
    if (nlhs /= 1) call mexerrmsgtxt(err2)

! get the input args
    tmp_pr=mxGetPr(prhs(5))
    call mxCopyPtrToReal8(tmp_pr,tmp,1)
    HH=int(tmp)

    tmp_pr=mxGetPr(prhs(4))
    call mxCopyPtrToReal8(tmp_pr,tmp,1)
    nbddl_test=int(tmp)

    if (nbddl_test/=nbddl) call mexerrmsgtxt(err5)

    tmp_pr=mxGetPr(prhs(3))
    call mxCopyPtrToReal8(tmp_pr,tmp,1)
    nbddlstab=int(tmp)

    N = nbddlstab*(2*HH+1)+2

! get the size of ddlstab
    mm = mxGetM(prhs(2))
    nn = mxGetN(prhs(2))
    if (min(mm,nn)/=1) call mexerrmsgtxt(err3)
    if (max(mm,nn) /= N) call mexerrmsgtxt(err4)

    ddlstab_pr=mxGetPr(prhs(2))
    

! Create matrix for the return arguments.
    plhs(1) = mxCreateDoubleMatrix(nbddl*(2*HH+1),nbddl*(2*HH+1),1)
    HILL_pr = mxGetPr(plhs(1))
    Hill_pi = mxGetPi(plhs(1))

    call calc_Hill(ddlstab_pr,N,HH,nbddlstab,HILL_pr, HILL_pi)

end subroutine mexFunction
