% Linear big operator bL(bU)
% with harmonic balance automatic computation of bL(bU)
 
function [L] = L(obj,U)

% User parameters
% ---------------

H = obj.H;   
Neq = obj.Neq;
om1c = obj.om1c;
om2c = obj.om2c;
mu1 = obj.mu1;
mu2 = obj.mu2;
  
% =============================================================================
% Harmonic balance automatic computation of bL(bU)
% which depends on pL0 
%
% Modifies only the call to pl0 to include the user parameters
% Lines marked with <--
% =============================================================================

indc= Neq+1:2*Neq:Neq*(2*H-1)+1;     % index of uic 
inds= 2*Neq+1:2*Neq:Neq*(2*H)+1;     % index of uis 

L=zeros(Neq*(2*H+1),1);
  
% L depends on pL

L(1:Neq)=pL(U(1:Neq),om1c,om2c,mu1,mu2);  % <--
for i=1:H
  L(Neq+2*(i-1)*Neq+1:2*Neq+2*(i-1)*Neq)  =pL(U(indc(i):indc(i)+Neq-1),om1c,om2c,mu1,mu2); % <--
  L(2*Neq+2*(i-1)*Neq+1:3*Neq+2*(i-1)*Neq)=pL(U(inds(i):inds(i)+Neq-1),om1c,om2c,mu1,mu2); % <--
end

% no phase equation
% =============================================================================

% ------------------------------------------
% Function pl, specific to the given problem
% ------------------------------------------

function [pL] = pL(U,om1c,om2c,mu1,mu2)

pL=zeros(4,1);
  
pL(1)=  U(3);      
pL(2)=  U(4);      
pL(3)=  -om1c*U(1) - mu1 * U(3);
pL(4)=  -om2c*U(2) - mu2 * U(4);
  

  
