% Quadratic big operator bQ(bU,bU)
% with harmonic balance automatic computation of bL(bU)
 
function [Q] = Q(obj,U,V)
 
% User parameters
% ---------------

H=obj.H;          
Neq=obj.Neq;   
beta1 = obj.beta1;
beta2 = obj.beta2;
  
% ==================================================================
% Harmonic balance automatic computation of bQ(bU,bU)
% which depends on pQ and pM 
%
% Modifies only the call to pQ and pM to include the user parameters
% Lines marked with <--
% ================================================================== 

indc=[Neq+1  :2*Neq:Neq*(2*H-1)+1];     % index of uic 
inds=[2*Neq+1:2*Neq:Neq*(2*H)+1  ];     % index of uis 

Q=zeros(Neq*(2*H+1),1);
  

% Harmonics 0
   
SOM=zeros(Neq,1);
   
for j=1:H
  SOM=SOM+pQ(U(indc(j):indc(j)+Neq-1),V(indc(j):indc(j)+Neq-1),beta1,beta2)+pQ(U(inds(j):inds(j)+Neq-1),V(inds(j):inds(j)+Neq-1),beta1,beta2);
end
Q(1:Neq)= 0.5*SOM + pQ(U(1:Neq),V(1:Neq),beta1,beta2) ; 
                    
% Harmonics 1,2,3,...
for ih=1:H
  SOMC=zeros(Neq,1);
  SOMS=zeros(Neq,1);
  
  % + terms
  for j=1:ih-1
    SOMC=SOMC+pQ(U(indc(ih-j):indc(ih-j)+Neq-1),V(indc(j):indc(j)+Neq-1),beta1,beta2) ...  % <--beta1,beta2
             -pQ(U(inds(ih-j):inds(ih-j)+Neq-1),V(inds(j):inds(j)+Neq-1),beta1,beta2);     % <--
          
    SOMS=SOMS+pQ(U(indc(ih-j):indc(ih-j)+Neq-1),V(inds(j):inds(j)+Neq-1),beta1,beta2) ...  % <--
             +pQ(U(inds(ih-j):inds(ih-j)+Neq-1),V(indc(j):indc(j)+Neq-1),beta1,beta2);     % <--
  end
  
  % - terms
  for j=ih+1:H
    SOMC=SOMC+pQ(U(indc(j):indc(j)+Neq-1),V(indc(j-ih):indc(j-ih)+Neq-1),beta1,beta2) ...  % <--
             +pQ(U(inds(j):inds(j)+Neq-1),V(inds(j-ih):inds(j-ih)+Neq-1),beta1,beta2) ...  % <--  
             +pQ(U(indc(j-ih):indc(j-ih)+Neq-1),V(indc(j):indc(j)+Neq-1),beta1,beta2) ...  % <--
             +pQ(U(inds(j-ih):inds(j-ih)+Neq-1),V(inds(j):inds(j)+Neq-1),beta1,beta2);     % <--
                 
    SOMS=SOMS+pQ(U(inds(j):inds(j)+Neq-1),V(indc(j-ih):indc(j-ih)+Neq-1),beta1,beta2) ...  % <--  
             -pQ(U(indc(j):indc(j)+Neq-1),V(inds(j-ih):inds(j-ih)+Neq-1),beta1,beta2) ...  % <--   
             -pQ(U(inds(j-ih):inds(j-ih)+Neq-1),V(indc(j):indc(j)+Neq-1),beta1,beta2) ...  % <--
             +pQ(U(indc(j-ih):indc(j-ih)+Neq-1),V(inds(j):inds(j)+Neq-1),beta1,beta2);     % <--                           
  end   
  % other terms
  Q(indc(ih):indc(ih)+Neq-1)=0.5*SOMC + pQ(U(indc(ih):indc(ih)+Neq-1),V(1:Neq),beta1,beta2)   ...   % Q(U,V) <--
                                      + pQ(U(1:Neq),V(indc(ih):indc(ih)+Neq-1),beta1,beta2)   ...   %  <--
                                      - ih*V(Neq*(2*H+1)+1)*pM(U(inds(ih):inds(ih)+Neq-1)) ;  % - lambda.M(U) <--
                                             
  Q(inds(ih):inds(ih)+Neq-1)=0.5*SOMS + pQ(U(inds(ih):inds(ih)+Neq-1),V(1:Neq),beta1,beta2)   ...   % Q(U,V) <--
                                      + pQ(U(1:Neq),V(inds(ih):inds(ih)+Neq-1),beta1,beta2)    ...  %  <--        
                                      + ih*V(Neq*(2*H+1)+1)*pM(U(indc(ih):indc(ih)+Neq-1));   % - lambda.M(U) <--
end   

% no phase equation
% =============================================================================


% ------------------------------------------
% Function pQ, specific to the given problem
% ------------------------------------------
 
function [pQ] = pQ(U,V,beta1,beta2)

pQ=zeros(4,1);
  
pQ(3)= -beta1*U(1)*V(2);
pQ(4)= -beta2*U(1)*V(1);
  
% ------------------------------------------
% Function pM, specific to the given problem
% ------------------------------------------ 
  
function [pM] = pM(U)

pM=U;