%  --------------------------------------------------------------------------
%  Calcul des solutions periodiques de l'equation autonome de Van-der-Pol
% + stabilit�
%  --------------------------------------------------------------------------
%
% L'�quation est donn� dans l'article JSV de Cochelin et Vergez
%
%   Model equation
%   --------------
%   The Van der Pol model is :
%   u'' - lambda (1-u^2) u' + u =  0  
%  u(t) is the unknown founction and  lambda a real parameter. 
%
%   we introduce   v = u'
%                  w = (1-u^2)
%                  r = v.w        
%
%  and we transform into 
%
%    u'  =     v
%    v'  =    -u   + lambda r 
%    0   = 1  -w                -  u^2
%    0   =     r                -  v w  
%
%    With  Z(t) = [ u , v , w , r]  (Neq=4) the system becomes
%  
%   -->   m(Z') = c0 +l0(Z) + lambda l1(Z) + q(Z,Z) 
%
%  
%  Harmonic balance
%  ----------------
%
%
%  Z(t) = Z0 + Z1c cos(omega t) + Z1s sin(omega t) + Z2c cos(2 Omega t) + Z2s sin(2 Omega t) + ... 
%              ...  + ZHc cos(H omega t) + ZHs sin(H omega t).
%
%  the balance of the harmonics leads to a large system of size  Neq*(2H+1)
%
%      C0    +     lambda C1 + L0(U)         + lambda L1(U) + Q(U,U) - omega M(U)= 0 
%                  
%
%     with    U=[Z0  Z1c Z1s  Z2c Z2s  ...  ZHs ]
%  
%
%  Final algebraic system for Manlab  bL0 + bL(bU) + bQ(bU,bU) =0 
%  --------------------------------------------------------------
%
%   bU = [ U , lambda, omega ]
%
%   bL0       = C0
%   bL(bu)    = L0(U) + lambda C1
%   bQ(bU,bU) = Q(U,U) + lambda L1(U) - omega M(U)
%
%
%--------------------------------------------------------------------------

 close all
 clear all
 clear classes
 
 addpath('~/MANLAB/SRC')
 
 manlabinit;  
 
% H = 20;
 H=4;
 Neq = 4;
 

 ML_problem  = VDP(H,Neq); 
 ML_problem = set_nbptstroncon(ML_problem,20);
 ML_problem = set_nech(ML_problem,3000);
 
 % starting point solution
 %lambda1=0;ML_Ustart   = zeros((2*H+1)*Neq+2,1); ML_Ustart(1:8)=[0 0 0.9 0 , 1 0.001 -0.001 0.001];  ML_Ustart(end-1)=lambda1; ML_Ustart(end)=1;

 ML_Ustart   = zeros((2*H+1)*Neq+2,1); 
 
 ML_Ustart(1:12)=[-0.000000000012836
                   0
  -0.000000001503225
  -0.000000000039208
   1.414213564039288
  -0.000000000003165
   0.000000000196622
  -0.000000000172419
  -0.000000000000000
  -1.414213564063415
  -0.000000000003984
  -0.000000000065913]; 

 ML_Ustart(end-1:end) =  [0.272254909363998 0.999999999926997];  


 % index of variable to be plotted
 %ML_dispvars = [  (2*H+1)*Neq+1  5 ; (2*H+1)*Neq+1  21 ; (2*H+1)*Neq+1 (2*H+1)*Neq+2 ];   
 ML_dispvars = [  (2*H+1)*Neq+1  1 ; (2*H+1)*Neq+1  5 ; (2*H+1)*Neq+1  9 ; (2*H+1)*Neq+1 (2*H+1)*Neq+2 ];
                 
 % Definition of the entry vector for the stability computations
 ML_varstab = [1 2 (2*H+1)*Neq+1 (2*H+1)*Neq+2 H];
 
 ML_stability = 1;
 ML_algostab = 2;
 ML_pointdisplay=0;
 ML_globaldisplay=0;

 manlabstart; 


